#!/bin/sh
# listdd.sh - called by /usr/bin/listdd_wrapper
# Purpose: list dynamic library dependencies
# Roger D. Grider © 2017 GPL3

export TEXTDOMAIN=listdd
export OUTPUT_CHARSET=UTF-8

export VER=2.1
export APPDIR="/usr/local/ListDD"
export PREFDIR="$APPDIR/preferences"
mkdir -p $PREFDIR

if [ "`which gtkdialog4`" ]; then
 GTKDIALOG=gtkdialog4
elif [ "`which gtkdialog3`" ]; then
 GTKDIALOG=gtkdialog3
else
 GTKDIALOG=gtkdialog
fi
export GTKDIALOG

# ----------
# FUNCTIONS
# ----------

case "$1" in
-summary)
if [ -f "$SEARCHPATH" ]; then
  ldd "$SEARCHPATH" | awk '{$NF=""; print $0}' | sed 's/=> not/=> not found/g' > /tmp/listdd.info
else
  echo "$(gettext 'No file selected')" > /tmp/listdd.info
fi
exit 0
;;

-complete)
if [ -f "$SEARCHPATH" ]; then
  ldd -v "$SEARCHPATH" | sed 's/^	//g' > /tmp/listdd.info
else
  echo "$(gettext 'No file selected')" > /tmp/listdd.info
fi
exit 0
;;

-unused)
if [ -f "$SEARCHPATH" ]; then
  ldd -u -r "$SEARCHPATH" | sed 's/^	//g' > /tmp/listdd.info
  [ ! "`cat /tmp/listdd.info`" ] && echo "$(gettext 'No unused dependencies')" > /tmp/listdd.info
else
  echo "$(gettext 'No file selected')" > /tmp/listdd.info
fi
exit 0
;;

-missing)
if [ -f "$SEARCHPATH" ]; then
  if [ ! "`grep 'not a dynamic' /tmp/listdd.info`" ]; then
   ldd -v "$SEARCHPATH" | grep 'not found' | sed -e 's/^[ \t]*//' > /tmp/listdd.info
   [ ! "`cat /tmp/listdd.info`" ] && echo "$(gettext 'No missing dependencies')" > /tmp/listdd.info
  fi
else
  echo "$(gettext 'No file selected')" > /tmp/listdd.info
fi
exit 0
;;

-scandir)
. /etc/DISTRO_SPECS
if [ "`which gtkdialog-splash`" ]; then
  gtkdialog-splash -close never -placement center -bg goldenrod -text " $(gettext 'Please wait, processing...') " &
elif [ "`which Xdialog`" ]; then
  Xdialog --title ListDD --no-buttons --infobox "\n     $(gettext 'Please wait, processing...')     \n" 0 0 180000 &
fi
XPID=$!
#adapted from wdlkmpx's checkdeps
  (
echo -e "${DISTRO_NAME}-${DISTRO_VERSION}\n$(date)\n"
SYSTEM=$(cat $PREFDIR/systemscan 2>/dev/null)
if [ "$SYSTEM" = "true" ]; then
  LIB32="" LIB64="" USRLIB32="" USRLIB64=""
  [ ! -L /lib32 ] && [ -d /lib32 ] && LIB32="/lib32"
  [ ! -L /lib64 ] && [ -d /lib64 ] && LIB64="/lib64"
  [ ! -L /usr/lib32 ] && [ -d /usr/lib32 ] && USRLIB32="/usr/lib32"
  [ ! -L /usr/lib64 ] && [ -d /usr/lib64 ] && USRLIB64="/usr/lib64"
  SEARCHPATH=$(echo "/bin /sbin /usr/bin /usr/sbin /usr/local/bin /usr/libexec /usr/games /lib ${LIB32} ${LIB64} /usr/lib ${USRLIB32} ${USRLIB64} /usr/local/lib" | tr -s '[:space:]')
fi
[ ! "$SEARCHPATH" ] && echo -e "$(gettext 'Scanned Dirs'): $(gettext 'none')\n" && kill $XPID && exit 1
echo -e "$(gettext 'Scanned Dirs'): ${SEARCHPATH}\n"
SCANNED=$(LANG=C find -H $SEARCHPATH -maxdepth 10 -type f | grep -Eav "/${DISTRO_ARCHDIR}/|^/lib/modules/|^/lib/firmware/|\.a$|\.h$|\.sh$|\.gz$|\.la$|\.py$|\.pm$|\.pl$|\.html$|\.rules$|pkgconfig|/X11/|/locale/|\.txt$" | wc -l)
echo -e "$(gettext 'Scanned Binary Files'): ${SCANNED}\n"
echo -e "$(gettext 'If detected, missing dependencies are listed here'):\n"
LANG=C
find -H $SEARCHPATH -maxdepth 10 -type f | sort | grep -Eav "/${DISTRO_ARCHDIR}/|^/lib/modules/|^/lib/firmware/|\.a$|\.h$|\.sh$|\.gz$|\.la$|\.py$|\.pm$|\.pl$|\.html$|\.rules$|pkgconfig|/X11/|/locale/|\.txt$" | \
while read -r FILE; do
  missinglibs=$(ldd -v "$FILE" 2>/dev/null | grep -a 'not found')
  [ "$missinglibs" ] && echo -e "${FILE}\n$missinglibs\n"
done
  ) > /tmp/listdd.info
kill $XPID
;;

-help)
 TEXT="$(gettext "ListDD is a graphical front-end for the ldd command which lists
dynamic dependencies of binary executables and library files.

Binary executables are typically found in /bin, /sbin, /usr/bin,
/usr/sbin, /libexec and /usr/games while many library files are
in /lib, /lib64, /usr/lib, /usr/lib64 and /usr/local/lib.

ListDD (ldd) does not support script executables, such as those
in /usr/local/bin, and simply defines them as 'not a dynamic'.

You can load files in ListDD with the 'File Selector' button,
or drag & drop from ROX-Filer. Alternatively, use the context
right-click menu of ROX-Filer or Thunar to select and load
an individual file for evaluation.
")"
 TEXT="$TEXT

$(gettext "The 'Summary' button provides a basic report of shared libraries
required by the selected file, and the 'Complete' button provides
a verbose report which includes symbol versioning information.

The 'Unused' button enumerates unused direct dependencies, and
the 'Missing' button reports dependencies that are required by
a program, but are not currently installed.

For installed applications, the 'Packages' button calls the
Puppy package manager to evaluate all required dependencies.

The 'Directory scan' button parses all binary executables and
library files in selected directories, or choose the 'System'
checkbox for a complete system-wide scan. Alternatively, use
the context right-click menu of ROX-Filer or Thunar to select
the directory and auto-scan files for missing dependencies.")"

 TEXT="$TEXT

$(gettext "ListDD-$VER
© 2017 GPL3
Roger D. Grider")"

echo "$TEXT" > /tmp/listdd.info
exit 0
;;

esac

# dirscan dialog
func_dirscan (){
export DIR_SCAN="<window title=\"$(gettext 'Directory Scan')\" icon-name=\"listdd\" resizable=\"false\">
 <vbox border-width=\"3\">
  <text use-markup=\"true\"><label>\"<b><span size='"'medium'"'>$(gettext 'Find Missing Dependencies')</span></b>\"</label></text>
  <vbox border-width=\"5\" space-expand=\"true\" space-fill=\"true\">
   <hbox width-request=\"400\">
    <text use-markup=\"true\"><label>\"<b>$(gettext 'Dir'):</b>\"</label></text>
    <entry tooltip-text=\" $(gettext 'Select a directory of binary executables or library files') \" fs-action=\"newdirectory\" fs-folder=\"/usr\" fs-title=\" $(gettext 'ListDD Directory Selector') \" secondary-icon-stock=\"gtk-clear\" secondary-icon-tooltip-text=\" $(gettext 'Clear entry and enable 
 system scan checkbox') \">
     <variable>SEARCHPATH</variable>
     <input>echo $SEARCHPATH</input>
     <action signal=\"changed\">echo false > $PREFDIR/systemscan</action>
     <action signal=\"changed\">refresh:SYSTEM</action>
     <action signal=\"changed\">disable:SYSTEM</action>
     <action signal=\"secondary-icon-release\">clear:SEARCHPATH</action>
     <action signal=\"secondary-icon-release\">enable:SYSTEM</action>
    </entry>
    <button relief=\"1\" height-request=\"30\" width-request=\"30\" tooltip-text=\" $(gettext 'Directory selector') \">	
     <variable>BUTTON</variable>
     <input file>$APPDIR/icons/directory.png</input><height>18</height><width>18</width>
     <action>fileselect:SEARCHPATH</action>
    </button>
   </hbox>
   <hbox homogeneous=\"true\">
    <checkbox tooltip-text=\" $(gettext 'Comprehensive system scan') \">
     <label>$(gettext 'System Scan')</label>
     <variable>SYSTEM</variable>
     <input>cat $PREFDIR/systemscan 2>/dev/null</input>
     <action>if true echo true > $PREFDIR/systemscan</action>
     <action>if true disable:SEARCHPATH</action>
     <action>if true disable:BUTTON</action>
     <action>if false echo false > $PREFDIR/systemscan</action>
     <action>if false enable:SEARCHPATH</action>
     <action>if false enable:BUTTON</action>
    </checkbox>
   </hbox>
  </vbox>

  <hbox space-expand=\"false\" space-fill=\"true\" homogeneous=\"true\">
    <button height-request=\"35\" tooltip-text=\" $(gettext 'Find missing dependencies') \">
     <label>$(gettext 'Search')</label>
     <input file stock=\"gtk-find\"></input>
     <action>touch /tmp/dirscan</action>
     <action>. listdd.sh -scandir &</action>
     <action type=\"exit\">quit_now</action>
    </button>
    <button height-request=\"35\">
     <label>$(gettext 'Cancel')</label>
     <input file stock=\"gtk-cancel\"></input>
     <action>touch /tmp/dirscan-no</action>
     <action>listdd.sh &</action>
     <action type=\"exit\">quit_now</action>
    </button>
  </hbox>
 </vbox>
</window>"
$GTKDIALOG --geometry +200+200 -p DIR_SCAN
unset DIR_SCAN
}
export -f func_dirscan

# ------------
# DEFINITIONS
# ------------

# define title-bar icon
ln -sf /usr/local/ListDD/icons/listdd.svg /usr/share/icons/hicolor/48x48/apps && gtk-update-icon-cache -f -i /usr/share/icons/hicolor 2>/dev/null

# auto-adjust header text color for light or dark GTK themes
if [ "`grep 'Stardust_dark' $HOME/.gtkrc-2.0 2>/dev/null`" ]; then #gold text for unique Stardust dark-grey-orange themes
  COLOR=#D7B740
else #light-blue text for common dark themes, otherwise medium blue
  [ "`grep -Ei 'black|dark|night' $HOME/.gtkrc-2.0 2>/dev/null`" ] && COLOR=#84aad9 || COLOR=#3272C0
fi
export COLOR

# define GUI fonts
echo "style \"specialmono\"
{
  font_name=\"Mono\"
}
widget \"*mono\" style \"specialmono\"
class \"GtkText*\" style \"specialmono\"

style \"specialframe\"
{
  font_name=\"Sans bold\"
#  fg[NORMAL]=\"grey40\"
}
widget \"*.GtkFrame.GtkLabel\" style \"specialframe\"
class \"*.GtkFrame.GtkLabel\" style \"specialframe\"" > /tmp/gtkrc_mono

export GTK2_RC_FILES=/tmp/gtkrc_mono:/root/.gtkrc-2.0

# define package-dependency check
if [ "`grep -a "package manager for Puppy Linux" /usr/sbin/petget`" ]; then
 CHECK_PKGDEPS=""
else
 CHECK_PKGDEPS="
   <button height-request=\"35\" width-request=\"100\" tooltip-text=\" $(gettext 'Check dependencies of installed packages') \" use-underline=\"true\">
    <label>$(gettext '_Packages')</label>
    <action>/usr/local/petget/check_deps.sh &</action>
   </button>"
fi

# define Dependencies dialog
if [ -f /tmp/dirscan ]; then #ldd folder
 rm -f /tmp/dirscan
elif [ -f /tmp/dirscan-no ]; then #cancel dirscan
 rm -f /tmp/dirscan-no
 SEARCHPATH=""
elif [ -f "$SEARCHPATH" ]; then #ldd file
 ldd "$SEARCHPATH" 2>/dev/null | awk '{$NF=""; print $0}' > /tmp/listdd.info
fi

if [ ! "`cat /tmp/listdd.info 2>/dev/null`" ]; then #default interface
 SEARCHPATH=""
 echo "$(gettext 'Select a binary executable or library file...')" > /tmp/listdd.info
fi

# ------------
# MAIN DIALOG
# ------------

export LISTDD="
<window title=\"LDD\" icon-name=\"listdd\" resizable=\"true\">
 <vbox>

  <text use-markup=\"true\" tooltip-text=\" $(gettext 'List Dynamic Dependencies') \"><label>\"<b><span size='"'x-large'"'>List</span><span size='"'x-large'"' color='"$COLOR"'>DD</span></b>\"</label></text>

  <frame $(gettext 'Dependencies')>
   <edit name=\"mono\" editable=\"false\">
    <variable>INFO</variable>
    <input file>/tmp/listdd.info</input>
    <height>360</height>
    <width>660</width>
   </edit>
  <hbox>
  <text use-markup=\"true\"><label>\"<b>$(gettext 'Path:')</b>\"</label></text>
  <entry height-request=\"30\" tooltip-text=\" $(gettext 'Select a binary executable or library file') \" fs-action=\"newfile\" fs-folder=\"/usr\" fs-title=\" $(gettext 'ListDD File Selector') \" secondary-icon-stock=\"gtk-clear\" secondary-icon-tooltip-text=\" $(gettext 'Clear entry') \">
    <variable>SEARCHPATH</variable>
    <input>echo $SEARCHPATH</input>
    <action signal=\"changed\">. listdd.sh -summary</action>
    <action signal=\"changed\">refresh:INFO</action>
    <action signal=\"secondary-icon-release\">clear:SEARCHPATH</action>
    <action signal=\"secondary-icon-release\">clear:INFO</action>
  </entry>
  <button relief=\"1\" height-request=\"30\" width-request=\"30\" tooltip-text=\" $(gettext 'File selector') \">	
    <input file>$APPDIR/icons/file.png</input><height>18</height><width>18</width>
    <action>fileselect:SEARCHPATH</action>
  </button>
  <button relief=\"1\" height-request=\"30\" width-request=\"30\" tooltip-text=\" $(gettext 'Directory scan') \">	
    <input file>$APPDIR/icons/directory.png</input><height>18</height><width>18</width>
    <action>echo false > $PREFDIR/systemscan</action>
    <action>SEARCHPATH="" ; func_dirscan &</action>
    <action>EXIT:quit_now</action>
  </button>
  <button relief=\"1\" height-request=\"30\" width-request=\"30\" tooltip-text=\" $(gettext 'Help') \">	
    <input file>$APPDIR/icons/help.svg</input><height>18</height><width>18</width>
    <action>. listdd.sh -help</action>
    <action>refresh:INFO</action>
  </button>
  </hbox>
  </frame>

  <hbox>
   <button has-focus=\"true\" height-request=\"35\" width-request=\"100\" tooltip-text=\" $(gettext 'Summary report') \" use-underline=\"true\">
    <label>$(gettext '_Summary')</label>
    <action>. listdd.sh -summary</action>
    <action>refresh:INFO</action>
   </button>
   <button height-request=\"35\" width-request=\"100\" tooltip-text=\" $(gettext 'Verbose report') \" use-underline=\"true\">
    <label>$(gettext '_Complete')</label>
    <action>. listdd.sh -complete</action>
    <action>refresh:INFO</action>
   </button>
   <button height-request=\"35\" width-request=\"100\" tooltip-text=\" $(gettext 'Unused direct dependencies') \" use-underline=\"true\">
    <label>$(gettext '_Unused')</label>
    <action>. listdd.sh -unused</action>
    <action>refresh:INFO</action>
   </button>
   <button height-request=\"35\" width-request=\"100\" tooltip-text=\" $(gettext 'Missing dependencies of selected file') \" use-underline=\"true\">
    <label>$(gettext '_Missing')</label>
    <action>. listdd.sh -missing</action>
    <action>refresh:INFO</action>
   </button>
   $CHECK_PKGDEPS
   <button height-request=\"35\" width-request=\"100\" use-underline=\"true\">
    <label>$(gettext 'E_xit')</label>
    <input file stock=\"gtk-close\"></input>
    <action>EXIT:quit_now</action>
   </button>
   <button space-expand=\"true\" space-fill=\"true\" visible=\"false\"><label>\" \"</label></button>
  </hbox>
 </vbox> 
</window>"

$GTKDIALOG --geometry +50+50 -p LISTDD > /dev/null 2>&1 </dev/null
unset LISTDD
rm -f /tmp/listdd.info /tmp/dirscan /tmp/dirscan-no 2>/dev/null
[ -r $HOME/.local/share/recently-used.xbel.bak ] && cp -f $HOME/.local/share/recently-used.xbel.bak $HOME/.local/share/recently-used.xbel

exit 0
